/*
 * Copyright (c) 1992, 1993
 *      The Regents of the University of California.  All rights reserved.
 *
 * This code is derived from software donated to Berkeley by
 * Jan-Simon Pendry.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *      @(#)rot13fs.h    8.3 (Berkeley) 8/20/94
 *
 * $Id: rot13fs.h,v 1.1.1.1 1998/11/05 21:06:33 ezk Exp $
 */

struct rot13fs_args {
  char *target;			/* Target of loopback  */
};

struct rot13fs_mount {
  struct mount *rot13fsm_vfs;
  struct vnode *rot13fsm_rootvp;	/* Reference to root rot13fs_node */
};

/* EZK ADDED */
/* for ioctl */
#define FIST_IOCTL_GET_DEBUG_VALUE	_IOR('F', 1,  int)
#define FIST_IOCTL_SET_DEBUG_VALUE	_IOW('F', 2,  int)
#define FIST_IOCTL_GET_NUM_VMA		_IOW('F', 3,  int)
#define FIST_IOCTL_SET_KEY		_IOW('F', 10, char [16])

#ifdef KERNEL
/*
 * A cache of vnode references
 * (This is a private structure per vnode->v_data)
 */
struct rot13fs_node {
  LIST_ENTRY(rot13fs_node) rot13fs_hash; /* Hash list */
  struct vnode *rot13fs_lowervp; /* VREFed once */
  struct vnode *rot13fs_vnode;	/* Back pointer */
};

extern int rot13fs_init __P((struct vfsconf * vfsp));
extern int rot13fs_node_create __P((struct mount *mp, struct vnode * target, struct vnode ** vpp));

#define	MOUNT_TO_ROT13FS_MOUNT(mp) ((struct rot13fs_mount *)((mp)->mnt_data))
#define	VP_TO_ROT13FS(vp) ((struct rot13fs_node *)(vp)->v_data)
#define	ROT13FS_TO_VP(xp) ((xp)->rot13fs_vnode)
#ifdef ROT13FS_DIAGNOSTIC
extern struct vnode *rot13fs_checkvp __P((struct vnode * vp, char *fil, int lno));
# define ROT13FS_VP_TO_LOWERVP(vp) rot13fs_checkvp((vp), __FILE__, __LINE__)
#else
# define ROT13FS_VP_TO_LOWERVP(vp) (VP_TO_ROT13FS(vp)->rot13fs_lowervp)
#endif /* ROT13FS_DIAGNOSTIC */

extern int rot13fs_bypass __P((struct vop_generic_args * ap));

extern vop_t **rot13fs_vnodeop_p;

/* XXX: EZK hacky stuff I have to do rather than change system headers */
typedef struct vnode vnode_t;
typedef struct vattr vattr_t;
typedef struct uio uio_t;
typedef struct iovec iovec_t;
typedef struct ucred cred_t;
typedef struct mount vfs_t;

#define	MIN(a,b)		(((a)<(b))?(a):(b))
#define	MAX(a,b)		(((a)>(b))?(a):(b))
#define VT_ROT13FS		VT_NULL
#define MOUNT_ROT13FS		MOUNT_NULL
#define kmem_alloc(size)	malloc((size), M_TEMP, M_WAITOK)
#define kmem_free(addr)		free((addr), M_TEMP)

extern int rot13_encode_block(char *func, int line, caddr_t base, int len, const vnode_t *vp);
extern int rot13_decode_block(char *func, int line, caddr_t base, int len, const vnode_t *vp);
extern void *kmem_zalloc(unsigned long size);
extern int fist_uiomove(caddr_t cp, int n, enum uio_rw rwflag, struct uio *uio);
extern void rot13_verify_lower_object(vnode_t *vp, char *fxn);

#ifdef FIST_DEBUG
extern void fist_dprint_internal(int level, char *str, ...);
extern void fist_print_vnode(char *, vnode_t *);
extern void fist_print_vfs(char *, vfs_t *);
/* extern void fist_print_uap(char *, struct mounta *); */
extern void fist_print_uio(const char *, uio_t *);
extern void fist_print_uios(const char *, uio_t *);
extern void fist_print_page(const char *fxn_name, vm_page_t pp);
/* extern void fist_print_pnp(char *, pathname_t *); */
/* extern void fist_print_pathname(char *fxn_name, struct pathname *pnp); */
extern int fist_get_debug_value(void);
extern int fist_set_debug_value(int val);
# define print_location() \
	fist_dprint(4, "%s %s:%d\n", __FUNCTION__, __FILE__, __LINE__)

# define printk printf
# define fist_dprint(level, str, args...) \
	fist_dprint_internal(level, str, ## args)
# define ASSERT(e)	((e) ? (void)0 : panic("ASSERTION \"%s\" FAILED at %s:%d", #e, __FILE__, __LINE__))

#else /* not FIST_DEBUG */

# define printk(format, args...)
# define fist_dprint(level, format, args...)
# define print_location()
# define fist_print_vnode(a, b)
# define fist_print_vfs(a, b)
# define fist_print_uap(a, b)
# define fist_print_page(a, b)
# define ASSERT(e)

#endif /* not FIST_DEBUG */

#endif /* KERNEL */
